import "dart:async";

import "package:http/http.dart" as http;
import "middleware.dart";
import "request.dart";
import "response.dart";
import "error.dart";

abstract class CoffeeConverter<T> {
  dynamic decode(dynamic data);

  dynamic encode(T data);
}

class CoffeeCrud<T> extends CoffeeHttpRequest {

  CoffeeConverter<T> _converter;

  bool _successfulRequest(int status) => status - 200 >= 0 && status - 200 < 100;

  CoffeeCrud(String url, this._converter,
      {List<CoffeeMiddleware> middlewares, Map<String, CoffeeHttpRequest> requests, http.Client client})
      : super(url, middlewares: middlewares, requests: requests, client: client) {
    this["all"] = new Get("", decoder: _converter.decode);
    this["read"] = new Get("/{id}", decoder: _converter.decode);
    this["create"] = new Post("", decoder: _converter.decode, encoder: _converter.encode);
    this["update"] = new Put("", decoder: _converter.decode, encoder: _converter.encode);
    this["delete"] = new Delete("/{id}", decoder: _converter.decode);
  }


  Future<T> create(T object) async {
    CoffeeResponse res = await this["create"].execute(body: object);
    if (_successfulRequest(res.statusCode)) {
      return res.decodedBody as T;
    }
    throw new CoffeeError(res, res.statusCode, res.body);
    return null;
  }

  Future<T> read(String id) async {
    CoffeeResponse res = await this["read"].execute(parameters: {"id": id});
    if (_successfulRequest(res.statusCode)) {
      return res.decodedBody as T;
    }
    throw new CoffeeError(res, res.statusCode, res.body);
    return null;
  }

  Future<T> update(T object) async {
    CoffeeResponse res = await this["update"].execute(body: object);
    if (_successfulRequest(res.statusCode)) {
      return res.decodedBody as T;
    }
    throw new CoffeeError(res, res.statusCode, res.body);
    return null;
  }

  Future<T> delete(String id) async {
    CoffeeResponse res = await this["delete"].execute(parameters: {"id": id});
    if (_successfulRequest(res.statusCode)) {
      return res.decodedBody as T;
    }
    throw new CoffeeError(res, res.statusCode, res.body);
    return null;
  }

  Future<List<T>> all() async {
    CoffeeResponse res = await this["all"].execute();
    if (_successfulRequest(res.statusCode)) {
      return res.decodedBody as List<T>;
    }
    throw new CoffeeError(res, res.statusCode, res.body);
    return null;
  }

}
